; =======================================================================================================================
; Title .........: MCFinclude.au3
; AutoIt Version : 3.3.14+
; Description ...: CodeCrypter target's UDFs
; Author.........: A.R.T. Jonkers (RTFC)
; Release........: 3.4
; Latest revision: 02 December 2021
; License........: free for personal use; free distribution allowed provided
;					the original author is credited; all other rights reserved.
; Tested on......: W10Pro/64
; Forum Link.....: http://www.autoitscript.com/forum/topic/155538-codecrypter-encrypt-your-script/
; Related to......: CodeScanner, MCF, CodeCrypter by RTFC
; Dependencies...:	MCFinclude.au3, by RTFC; part of the CodeScannerCrypter bundle
; ===============================================================================================================================
; Summary (Wotsit?)
;
; This Codecrypter utility allows you to copy and transfer a full MCF
;	encryption profile (CodeCrypter's $CCkey array, filled with all
;	responses from a chosen target environment) from one physical machine
;	to another. Codecrypter can read this (itself encrypted) file to perform
;	multi-key encryption based upon the provided complete target profile.
;
; No guarantees, no warranties, no liability for damages, no refunds.
; Use at your own risk.
;
;============================================================================================
;	Application
;
; 1.	On the host machine (the one to be used for encryption), open
;		MCFinclude.au3 and edit the $CCkey array as you see fit. Save it.
;
; 2.	Open StoreCCprofile.au3 and compile to exe.
;		The executable now contains the latest version of MCFinclude's $CCkey array.
;		Copy/Move this exe to a USB stick.
;
; 3.	On the target machine, insert the USB stick and run StoreCCkey.exe
;		from the stick! Do NOT copy this exe to the target machine!
;		The exe retrieves all $CCkey contents from the target machine,
;		storing these in encrypted file "profile.mcf" on the USB
;
; 4.	On the host machine, copy profile.mcf to the encryption work directory.
;		Start CodeCrypter, and perform all required steps prior to encryption.
;
; 5.	Under Tab Encrypt, press button [Decryptor].
;		In the Decryptor UI, press button [Profile].
;		Select other encryption options as normal, and encrypt.
;
;	Note: the resulting output will NOT run on the host machine,
;		and can only be tested in the target environment.
;
; * NB the locally-stored profile is read in UDF _EncryptArrays() in MCF.au3
;
; ===============================================================================================================================
#include-once
#NoTrayIcon

#include <Crypt.au3>
#include <File.au3>
#include <FileConstants.au3>
#include <MsgBoxConstants.au3>

#include ".\MCFinclude.au3"			; default MCFinclude, contains $MCFprofile ([path+]filename)

Global Const $iAlgorithm=$CALG_AES_256

Global $fh=FileOpen($MCFprofile,2+8+$FO_UTF8)
_FileWriteFromArray($fh,$CCkey)
FileClose($fh)

Global $fh=FileOpen($MCFprofile,$FO_UTF8)
Global $test=FileReadToArray($fh)
FileClose($fh)

If UBound($CCkey)<>UBound($test) Then
	msgbox($MB_ICONERROR,"SToreCCprofile", "Error: array size mismatch" & @CR & UBound($CCkey) & " vs " &UBound($test))
	FileDelete($MCFprofile)
	Exit(1)
EndIf

For $cc=0 To UBound($test)-1
	If $test[$cc]="" Then $test[$cc]=Null
	If $CCkey[$cc]<>$test[$cc] Then
		msgbox($MB_ICONERROR,"SToreCCprofile", "Error: array content mismatch in line " & $cc)
		FileDelete($MCFprofile)
		Exit(2)
	EndIf
next

$tmpfilenc=_TempFile()
_Crypt_EncryptFile($MCFprofile, $tmpfilenc, $CCkey[0], $iAlgorithm)
$err=@error
FileDelete($MCFprofile)
If $err Then
    Switch $err
        Case 1
            MsgBox($MB_SYSTEMMODAL, "SToreCCprofile Error", "Failed to create the key.")
        Case 2
            MsgBox($MB_SYSTEMMODAL, "SToreCCprofile Error", "Couldn't open the source file.")
        Case 3
            MsgBox($MB_SYSTEMMODAL, "SToreCCprofile Error", "Couldn't open the destination file.")
        Case 4 Or 5
            MsgBox($MB_SYSTEMMODAL, "SToreCCprofile Error", "Encryption error.")
	EndSwitch
	Exit(3)
EndIf

FileMove($tmpfilenc, $MCFprofile, $FC_OVERWRITE)
MsgBox($MB_ICONINFORMATION,"SToreCCprofile","Profile Saved to" & @CR & $MCFprofile)

$test=Null
$CCkey=Null
Exit(0)
